#!/usr/bin/python3

import itertools
import os
import sys
import xml.etree.ElementTree as ET

def toposort(graph):
    order = {}

    def visit(node, depth):
        if order.get(node, -1) < depth:
            order[node] = depth
            for dep in graph[node]:
                visit(dep, depth + 1)

    for node in graph.keys():
        visit(node, 0)

    output = [(-depth, node) for node, depth in order.items()]
    output.sort()
    return [node for _, node in output]

debian_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
package_dir = os.path.dirname(debian_dir)

packages = [name for name in os.listdir(package_dir) if name.startswith("ament_cmake")]
packages.sort()

graph = {}

for name in packages:
    xml = ET.parse(os.path.join(package_dir, name, "package.xml"))
    deps = set()

    for d in itertools.chain(xml.findall("./buildtool_depend"), xml.findall("./buildtool_export_depend"), xml.findall("./build_depend"), xml.findall("./build_export_depend"), xml.findall("./test_depend"), xml.findall("./depend")):
        if d.text.startswith("ament_cmake"):
            deps.add(d.text)
    graph[name] = deps

order = toposort(graph)

sys.stdout.write("AMENT_MODULES = \\\n    ")
sys.stdout.write(" \\\n    ".join(order))
sys.stdout.write("\n\n")

for name in order:
    deps = ", ".join(sorted(list(graph[name])))
    sys.stdout.write (f"# {name} -> {deps}\n")
